/* cupsRaster.c test CUPS raster ABI interfaces */
/* 9/22/2007 Stew Benedict stewb@linux-foundation.org */

/* interfaces:
cupsRasterClose
cupsRasterOpen
cupsRasterReadHeader2
cupsRasterReadPixels
cupsRasterWriteHeader2
cupsRasterWritePixels
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <tet_api.h>
#include <cups/raster.h>

const char rfile[12] = "test.raster";
const char wfile[13] = "testw.raster";

void test_cleanup();
void (*tet_startup)() = NULL;
void (*tet_cleanup)() = test_cleanup;
void test1();
void test2();
void test3();
void test4();
void test5();
void test6();

struct tet_testlist tet_testlist[] = 
{ 
    {test1,1},
    {test2,2},
    {test3,3},
    {test4,4},
    {test5,5},
    {test6,6},
	{NULL,0} 
};

FILE* openRasterFile(const char* rfile, char* mode)
{
    FILE                  *fp;

    if ((fp = fopen(rfile, mode)) == NULL){
        tet_printf("Cannot open %s in mode %s", rfile, mode);
        return NULL;
    } else {
        return fp;
    }
}

void test_cleanup()
{
    unlink(wfile);
}

void test1()
{
    FILE                  *fp;
    cups_raster_t         *r;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsRasterOpen is called on an open file descriptor, the function returns a raster stream");

    if ((fp = openRasterFile(rfile, "r")) == NULL){
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((r = cupsRasterOpen(fileno(fp), CUPS_RASTER_READ)) == NULL) {
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    cupsRasterClose(r);
    fclose(fp);
}

void test2()
{
    FILE                  *fp;
    cups_raster_t         *r;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsRasterClose is called on an open raster stream, the function shall complete with no error");

    if ((fp = openRasterFile(rfile, "r")) == NULL){
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((r = cupsRasterOpen(fileno(fp), CUPS_RASTER_READ)) == NULL) {
        tet_printf("cupsRasterOpen failed");
        tet_result(TET_UNRESOLVED);
    }

    errno = 0;
    cupsRasterClose(r);
    if (errno) {
        tet_printf("cupsRasterClose returned %d", errno);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    fclose(fp);
}

void test3()
{
    FILE                  *fp;
    cups_raster_t         *r;
    cups_page_header2_t    h;
    int                   result;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsRasterReadHeader2 is called on an open raster stream, the function shall return 1");

    if ((fp = openRasterFile(rfile, "r")) == NULL){
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((r = cupsRasterOpen(fileno(fp), CUPS_RASTER_READ)) == NULL) {
        tet_printf("cupsRasterOpen failed");
        tet_result(TET_UNRESOLVED);
        return;
    }
    
    if ((result = cupsRasterReadHeader2(r, &h)) == 1) {
        tet_result(TET_PASS);
    } else {
        tet_printf("cupsRasterReadHeader2 returned %d, expected 1", result);
        tet_result(TET_FAIL);        
    }
    cupsRasterClose(r);
    fclose(fp);    
}

void test4()
{
    FILE                  *fp;
    cups_raster_t         *r;
    cups_page_header2_t     h;
    unsigned char          p[65536];
    unsigned               result;
    unsigned               len;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsRasterReadPixels is called on an open raster stream, the function shall return the number of bytes read");

    if ((fp = openRasterFile(rfile, "r")) == NULL){
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((r = cupsRasterOpen(fileno(fp), CUPS_RASTER_READ)) == NULL) {
        tet_printf("cupsRasterOpen failed");
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((result = cupsRasterReadHeader2(r, &h)) != 1) {
        tet_result(TET_UNRESOLVED);
    }

    len = h.cupsBytesPerLine;
    if ((result = cupsRasterReadPixels(r, p, len)) != len) {
        tet_printf("cupsRasterReadPixels returned %u, expecting %u", result, len);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    cupsRasterClose(r);
    fclose(fp);
}

void test5()
{
    FILE                  *fp;
    cups_raster_t         *r;
    cups_page_header2_t     h;
    int                    result;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsRasterWriteHeader2 is called on an open raster stream, the function shall return 1");

    if ((fp = openRasterFile(rfile, "r")) == NULL){
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((r = cupsRasterOpen(fileno(fp), CUPS_RASTER_READ)) == NULL) {
        tet_printf("cupsRasterOpen failed");
        tet_result(TET_UNRESOLVED);
        return;
    }
    
    if ((result = cupsRasterReadHeader2(r, &h)) != 1) {
        tet_printf("cupsRasterReadHeader2 failed");
        tet_result(TET_UNRESOLVED);
    }
    cupsRasterClose(r);
    fclose(fp);

    if ((fp = openRasterFile(wfile, "w")) == NULL){
        tet_result(TET_UNRESOLVED);
        return;
    }
   
    if ((r = cupsRasterOpen(fileno(fp), CUPS_RASTER_WRITE)) == NULL) {
        tet_printf("cupsRasterOpen failed");
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((result = cupsRasterWriteHeader2(r, &h)) == 1) {
        tet_result(TET_PASS);
    } else {
        tet_printf("cupsRasterWriteHeader2 returned %d, expected 1", result);
        tet_result(TET_FAIL);        
    }
    cupsRasterClose(r);
    fclose(fp);
}

void test6()
{
    FILE                  *fp;
    cups_raster_t         *r;
    cups_page_header2_t     h;
    unsigned char          p[65536];
    unsigned               result;
    unsigned               len;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsRasterWritePixels is called on an open raster stream, the function shall return the number of bytes written");

    if ((fp = openRasterFile(rfile, "r")) == NULL){
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((r = cupsRasterOpen(fileno(fp), CUPS_RASTER_READ)) == NULL) {
        tet_printf("cupsRasterOpen failed");
        tet_result(TET_UNRESOLVED);
        return;
    }
    
    if ((result = cupsRasterReadHeader2(r, &h)) != 1) {
        tet_printf("cupsRasterReadHeader2 failed");
        tet_result(TET_UNRESOLVED);
    }

    len = h.cupsBytesPerLine;
    if ((result = cupsRasterReadPixels(r, p, len)) != len) {
        tet_printf("cupsRasterReadPixels returned %u, expecting %u", result, len);
        tet_result(TET_UNRESOLVED);
    }
    cupsRasterClose(r);
    fclose(fp);

    if ((fp = openRasterFile(wfile, "w")) == NULL){
        tet_result(TET_UNRESOLVED);
        return;
    }
   
    if ((r = cupsRasterOpen(fileno(fp), CUPS_RASTER_WRITE)) == NULL) {
        tet_printf("cupsRasterOpen failed");
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((result = cupsRasterWriteHeader2(r, &h)) != 1) {
        tet_printf("cupsRasterWriteHeader2 returned %d, expected 1", result);
        tet_result(TET_UNRESOLVED);        
    }

    if ((result = cupsRasterWritePixels(r, p, len)) != len) {
        tet_printf("cupsRasterWritePixels returned %u, expecting %u", result, len);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    cupsRasterClose(r);
    fclose(fp);
}

